//  Demo 7: SmoothlyMovingSquare.java
//
//     Another Simple Java Application to Use the Square Class
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999
//
// A Square smoothly moving diagonally across the screen.
// Illustrates Graphics double buffering to smooth animation,
// in an application.

// This is basically the same program as the preceeding demo,
// but it is an application rather than an Applet.

// The two step process for creating a Graphics buffer object to 
// draw on is more complex in an application.  First, the 
// application object has to exist before createImage can be
// successfully called, so we have to add an init() method which
// must be called in main().  Second, we have to wait while the
// system does its work creating an Image object, or createImage
// will return a null pointer.  As you know by now, this is
// BAD! BAD! BAD!

import java.awt.*;
import Square;
import CloseAndExit;

public class SmoothlyMovingSquare extends Frame {

   Square s;  // our Square

   Graphics gBuffer;   // Graphics buffer for off-screen drawing
   Image imageBuffer;  // need an Image object to create our
                       // own graphics object, for obscure
                       // Java reasons that only Sun understands


   // Constructor: as usual, initializes variables
   public SmoothlyMovingSquare() {

      s = new Square(50, 50, 25, Color.red);  // initialize object
                                              // state variable s

     addWindowListener(new CloseAndExit());   // as usual

   } // Constructor SmoothlyMovingSquare()

   // init(): creates the Graphics buffer object.  Can't do
   //         this in our Constructor, because we actually
   //         have to exist before the createImage method will
   //         work.
   public void init() {

     // create the gBuffer Graphics buffer object
     //   two step process, more Java obscurity
     imageBuffer = createImage(400, 400);  // 400 by 400 pixels

     // we have to wait while the system creates the imageBuffer,
     // or we will simply get a null pointer.
     while(imageBuffer == null) {

        try {
          Thread.sleep(50);  // 50 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }
     }

     // and finally, we can get our graphics buffer!
     gBuffer = imageBuffer.getGraphics();

     repaint();

   } // init()

   public void paint(Graphics g) {

     // don't do anything unless imageBuffer is non-null
     if (imageBuffer == null) return;

     Color backgroundColor = getBackground();

     // Here comes the animation!
     for(int i = 1; i <= 200; i++) {

        // move -- do drawing offscreen on the Graphics
        //         buffer object
        s.draw(gBuffer, backgroundColor);  // hide square
        s.move(1, 1);
        s.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        // To pause, so the animation doesn't go too fast, use
        // the sleep() method of the Thread class.  Unfortunately,
        // it throws an InterruptedException, so we have to embed
        // our simple pause statement in a try -- catch clause.
        try {
          Thread.sleep(20);  // 20 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }
      }

   } // paint

   public static void main(String[] args) {

     SmoothlyMovingSquare sms = new SmoothlyMovingSquare();


     sms.setSize(400, 400);
     sms.setVisible(true);
     sms.init();

   } // main()

} // class SmoothlyMovingSquare
        
